@@ -27,8 +27,8 @@
  *****************************************************************************/
 typedef struct argument
 {
-  char *var;
-  char *value;
+     char *var;
+     char *value;
 } argument_t;
 
 typedef struct data
@@ -37,6 +37,7 @@ typedef struct data
      char *displayname;
      NPWindow windata;
      int pid;
+     int fd;
      int repeats;
      int flags;
      char *command;
@@ -80,6 +81,7 @@ typedef struct
 /*****************************************************************************
  * Global variables
  *****************************************************************************/
+static char *config_fname;
 static char *helper_fname;
 static char *controller_fname;
 static struct handle *first_handle;
@@ -162,14 +164,15 @@ static void run(NPP instance, const char *file)
      if ((THIS->flags & H_CONTROLS) && (THIS->mode == NP_FULL))
 	  THIS->flags &= ~(H_CONTROLS | H_SWALLOW | H_FILL);
 
-     snprintf(buffer, sizeof(buffer), "%d,%d,%lu,%d,%d,%d,%d",
-	     THIS->flags,
-	     THIS->repeats,
-	     (unsigned long int) THIS->windata.window,
-	     (int) THIS->windata.x,
-	     (int) THIS->windata.y,
-	     (int) THIS->windata.width,
-	     (int) THIS->windata.height);
+     snprintf(buffer, sizeof(buffer), "%d,%d,%d,%lu,%d,%d,%d,%d",
+	      THIS->flags,
+	      THIS->repeats,
+	      THIS->fd,
+	      (unsigned long int) THIS->windata.window,
+	      (int) THIS->windata.x,
+	      (int) THIS->windata.y,
+	      (int) THIS->windata.width,
+	      (int) THIS->windata.height);
 
      D("Executing helper: %s %s %s %s %s %s\n",
        helper_fname,
@@ -351,6 +354,7 @@ static int parse_flags(char **x, command_t *commandp)
 	  { "noisy",		H_NOISY 	},
 	  { "embed",            H_EMBED         },
 	  { "noembed",          H_NOEMBED       },
+	  { "hidden",           H_HIDDEN        },
 	  { NULL, 		0 		}
      };
      flag_t *f;
@@ -369,15 +373,9 @@ static int parse_flags(char **x, command_t *commandp)
 	       if ((*x = get_winname(*x, "swallow", commandp)))
 		    return 1;
 	  }
-	  if (match(*x, "event_swallow"))
-	  {
-	       commandp->flags |= H_EVENT_SWALLOW;
-	       if ((*x = get_winname(*x, "event_swallow", commandp)))
-		    return 1;
-	  }
 	  if (match(*x, "controls"))
 	  {
-	       commandp->flags |= H_CONTROLS | H_EVENT_SWALLOW | H_FILL;
+	       commandp->flags |= H_CONTROLS | H_FILL | H_SWALLOW | H_HIDDEN;
 	       commandp->winname = strdup("mozplugger-controller");
 	       *x += 8;
 	       return 1;
@@ -574,10 +572,48 @@ static int find_helper_file(char *basename, int (*cb)(char *,void *data), void *
 
 static int read_config_cb(char *fname, void *data)
 {
-     FILE *f = fopen(fname,"r");
+     int m4out[2];
+     int pid;
+     int fd;
+     FILE *f;
+
+     D("READ_CONFIG(%s)\n", fname);
+
+     fd = open(fname, O_RDONLY);
+     if (fd < 0) return 0;
+
+     if (pipe(m4out) < 0)
+     {
+	  perror("pipe");
+	  return 0;
+     }
+
+     if ((pid = fork()) == -1)
+	  return 0;
+
+     if (!pid)
+     {
+	  dup2(fd, 0);
+	  dup2(m4out[1], 1);
+
+	  close(fd);
+	  close(m4out[1]);
+
+	  execlp("m4", "m4", NULL);
+	  fprintf(stderr, "MozPlugger: Failed to execute m4.\n");
+	  exit(1);
+     }
+
+     close(fd);
+     close(m4out[1]);
+
+     f = fdopen(m4out[0], "r");
      if (!f) return 0;
      read_config(f);
      fclose(f);
+     
+     waitpid(pid, 0, 0);
+     config_fname = strdup(fname);
      return 1;
 }
 
@@ -591,10 +627,10 @@ static int find_plugger_helper_cb(char *fname, void *data)
 
 static int find_plugger_controller_cb(char *fname, void *data)
 {
-  struct stat buf;
-  if(stat(fname, &buf)) return 0;
-  controller_fname = strdup(fname);
-  return 1;
+     struct stat buf;
+     if(stat(fname, &buf)) return 0;
+     controller_fname = strdup(fname);
+     return 1;
 }
 
 /*****************************************************************************
@@ -612,17 +648,25 @@ static void do_read_config(void)
 	  return;
      }
 
-     if (find("mozplugger-helper")) helper_fname = "mozplugger-helper";
-     find_helper_file("mozplugger-helper", find_plugger_helper_cb, 0);
-
-     if (!helper_fname)
+     if (!find_helper_file("mozplugger-helper", find_plugger_helper_cb, 0))
      {
-	  fprintf(stderr, "MozPlugger: Unable to find the mozplugger helper.\n");
-	  return;
+	  if (find("mozplugger-helper")) {
+	       helper_fname = "mozplugger-helper";
+	  } else { 
+	       fprintf(stderr, "MozPlugger: Unable to find mozplugger-helper.\n");
+	       return;
+	  }
+     }
+
+     if (!find_helper_file("mozplugger-controller", find_plugger_controller_cb, 0))
+     {
+	  if (find("mozplugger-controller")) {
+	       controller_fname = "mozplugger-controller";
+	  } else {
+	       fprintf(stderr, "MozPlugger: Unable to find mozplugger-controller.\n");
+	       return;
+	  }
      }
-     
-     if (find("mozplugger-controller")) controller_fname = "mozplugger-controller";
-     find_helper_file("mozplugger-controller", find_plugger_controller_cb, 0);
 
      D("do_read_config done\n");
 }
@@ -633,21 +677,21 @@ static void do_read_config(void)
  *****************************************************************************/
 static int safeURL(char* url)
 {
-    int  i = 0; 
-    int  len = strlen(url);
+     int  i = 0; 
+     int  len = strlen(url);
     
-    if (url[0] == '/')
-	 return 0;
+     if (url[0] == '/')
+	  return 0;
 
-    for(i = 0; i < len; i++)
-    {
-        if (url[i] == '`' || url[i] == ';')
-        {
-            /* Somebody's trying to do something naughty. */
-            return 0;
-        }
-    }
-    return 1;
+     for(i = 0; i < len; i++)
+     {
+	  if (url[i] == '`' || url[i] == ';')
+	  {
+	       /* Somebody's trying to do something naughty. */
+	       return 0;
+	  }
+     }
+     return 1;
 }
 
 /*****************************************************************************
@@ -765,10 +809,11 @@ char *NPP_GetMIMEDescription(void)
 }
 
 /*****************************************************************************
- * Let mozilla know things about mozplugger.
+ * Let Mozilla know things about mozplugger.
  *****************************************************************************/
 NPError NPP_GetValue(void *instance, NPPVariable variable, void *value)
 {
+     static char desc_buffer[8192];
      NPError err = NPERR_NO_ERROR;
 
      D("Getvalue %d\n", variable);
@@ -784,17 +829,26 @@ NPError NPP_GetValue(void *instance, NPPVariable variable, void *value)
 
      case NPPVpluginDescriptionString:
 	  D("GET Plugin description\n");
-	  *((char **)value) =
-	       "MozPlugger version "
-	       VERSION
-	       ", written by "
-	       "<a href=http://fredrik.hubbe.net/>Fredrik H&uuml;binette</a> "
-	       "<a href=mailto:hubbe@hubbe.net>&lt;hubbe@hubbe.net&gt</a> "
-	       "and Louis Bavoil "
-	       "<a href=mailto:louis@bavoil.net>&lt;louis@bavoil.net&gt</a>.<br>"
-	       "For documentation on how to configure mozplugger, "
-	       "check the man page. (type <tt>man&nbsp;mozplugger</tt>)"
-	       ;
+	  snprintf(desc_buffer, sizeof(desc_buffer),
+		   "MozPlugger version "
+		   VERSION
+		   ", written by "
+		   "<a href=http://fredrik.hubbe.net/>Fredrik H&uuml;binette</a> "
+		   "<a href=mailto:hubbe@hubbe.net>&lt;hubbe@hubbe.net&gt</a> "
+		   "and Louis Bavoil "
+		   "<a href=mailto:louis@bavoil.net>&lt;louis@bavoil.net&gt</a>.<br>"
+		   "For documentation on how to configure mozplugger, "
+		   "check the man page. (type <tt>man&nbsp;mozplugger</tt>)"
+		   " <table> "
+		   " <tr><td>Configuration file:</td><td>%s</td></tr> "
+		   " <tr><td>Helper binary:</td><td>%s</td></tr> "
+		   " <tr><td>Controller binary:</td><td>%s</td></tr> "
+		   " </table> "
+		   "<br clear=all>",
+		   config_fname ? config_fname : "Not found!",
+		   helper_fname ? helper_fname : "Not found!",
+		   controller_fname ? controller_fname : "Not found!");
+	  *((char **)value) = desc_buffer;
 	  break;
 
      default:
@@ -827,15 +881,15 @@ static int my_atoi(char *s, int my_true, int my_false)
  * that there might be several instances going at one time.
  *****************************************************************************/
 NPError NPP_New(NPMIMEType pluginType,
-		 NPP instance,
-		 uint16 mode,
-		 int16 argc,
-		 char* argn[],
-		 char* argv[],
-		 NPSavedData* saved)
+		NPP instance,
+		uint16 mode,
+		int16 argc,
+		char* argn[],
+		char* argv[],
+		NPSavedData* saved)
 {
      int e;
-     int src_attr = -1;
+     int src_idx = -1;
 
      D("NEW (%s)\n", pluginType);
 
@@ -857,8 +911,8 @@ NPError NPP_New(NPMIMEType pluginType,
 
      THIS->windata.window = 0;
      THIS->display = NULL;
-     THIS->displayname = NULL;
      THIS->pid = -1;
+     THIS->fd = -1;
      THIS->repeats = 1;
      THIS->autostart = 1;
      THIS->mode = mode;
@@ -883,7 +937,7 @@ NPError NPP_New(NPMIMEType pluginType,
 	  /* get the index of the src attribute */
 	  else if (!strcasecmp("src", argn[e]))
 	  {
-	       src_attr = e;
+	       src_idx = e;
 	  }
 	  /* copy the flag to put it into the environement later */
 	  D("VAR_%s=%s\n", argn[e], argv[e]);
@@ -900,17 +954,16 @@ NPError NPP_New(NPMIMEType pluginType,
           D("arg %d(%s): %s\n",e,argn[e],argv[e]);
           if ((!strcasecmp("href", argn[e]) ||
 	       !strcasecmp("qtsrc", argn[e])) &&
-	      src_attr > -1 &&
-	      !THIS->href)
+	      src_idx > -1 && !THIS->href)
           {
 	       if (!(THIS->href = strdup(argv[e])))
 		    return NPERR_OUT_OF_MEMORY_ERROR;
           }
      }
 
-     if (src_attr > -1)
+     if (src_idx > -1)
      {
-	  char *url = argv[src_attr];
+	  char *url = argv[src_idx];
 	  if (!strncmp(url, "mms://", 6) || !strncmp(url, "mmst://", 7))
 	  {
 	       D("Detected MMS\n");
@@ -941,6 +994,9 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
  	  if (THIS->pid > 0)
 	       my_kill(-THIS->pid);
 
+	  if (THIS->fd > 0)
+	       close(THIS->fd);
+
 	  for (e = 0; e < THIS->num_arguments; e++)
 	  {
 	       free((char *)THIS->args[e].var);
@@ -962,6 +1018,48 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
      return NPERR_NO_ERROR;
 }
 
+/*****************************************************************************
+ * Check that no child is already running before forking one.
+ *****************************************************************************/
+static void new_child(NPP instance, const char* fname)
+{
+     int pipe[2];
+
+     D("NEW_CHILD(%s)\n", fname);
+
+     if (!instance || !fname)
+	  return;
+
+     if (THIS->pid != -1)
+	  return;
+
+     if (socketpair(AF_UNIX, SOCK_STREAM, 0, pipe) < 0)
+     {
+	  NPN_Status(instance, "MozPlugger: Failed to create a pipe!");
+	  return;
+     }
+
+     if ((THIS->pid = my_fork(instance)) == -1)
+	  return;
+    
+     if (!THIS->pid)
+     {
+	  if (!find_command(instance, 1) && !find_command(instance, 0))
+	       _exit(EX_UNAVAILABLE);
+
+	  THIS->fd = pipe[1];
+	  close(pipe[0]);
+
+	  D("CHILD RUNNING run() [2]\n");
+	  run(instance, fname);
+
+	  _exit(EX_UNAVAILABLE);
+     } else {
+	  THIS->fd = pipe[0];
+	  close(pipe[1]);
+     }
+}
+
 /*****************************************************************************
  * Open a new stream.
  * Each instance can only handle one stream at a time.
@@ -992,8 +1090,8 @@ NPError NPP_NewStream(NPP instance,
       *  anything dirty. */
      if (THIS->href != NULL && safeURL(THIS->href))
      {
-         D("Replacing SRC with HREF... \n");
-         stream->url = THIS->href;
+	  D("Replacing SRC with HREF... \n");
+	  stream->url = THIS->href;
      }
 
      D("Mime type %s\n", type);
@@ -1001,7 +1099,7 @@ NPError NPP_NewStream(NPP instance,
      
      if (!find_command(instance, 1) && !find_command(instance, 0))
      {
-	  NPN_Status(instance, "No appropriate application found !");
+	  NPN_Status(instance, "MozPlugger: No appropriate application found !");
 	  return NPERR_GENERIC_ERROR;
      }
 
@@ -1012,19 +1110,7 @@ NPError NPP_NewStream(NPP instance,
      {
 	  *stype = NP_NORMAL;
 	  tempfile = 0;
-
-	  if ((THIS->pid = my_fork(instance)) == -1)
-	  {
-	       NPN_Status(instance, "Streamer: My_Fork failed!");
-	       return NPERR_GENERIC_ERROR;
-	  }
-    
-	  if (!THIS->pid)
-	  {
-	       D("CHILD RUNNING run() [1]\n");
-	       run(instance, stream->url);
-	       _exit(EX_UNAVAILABLE);
-	  }
+	  new_child(instance, stream->url);
      } else {
 	  *stype = NP_ASFILEONLY;
 	  tempfile = 1;
@@ -1044,31 +1130,6 @@ int32 NPP_WriteReady(NPP instance, NPStream *stream)
      return 0x7fffffff;
 }
 
-/*****************************************************************************
- * Check that no child is already running before forking one.
- *****************************************************************************/
-static void new_child(NPP instance, const char* fname)
-{
-     if (!instance || !fname)
-	  return;
-
-     if (THIS->pid != -1)
-	  return;
-
-     if ((THIS->pid = my_fork(instance)) == -1)
-	  return;
-    
-     if (!THIS->pid)
-     {
-	  if (!find_command(instance, 1) && !find_command(instance, 0))
-	       _exit(EX_UNAVAILABLE);
-
-	  D("CHILD RUNNING run() [2]\n");
-	  run(instance, fname);
-	  _exit(EX_UNAVAILABLE);
-     }
-}
-
 /*****************************************************************************
  * Called after NPP_NewStream if *stype = NP_ASFILEONLY.
  *****************************************************************************/
@@ -1080,44 +1141,6 @@ void NPP_StreamAsFile(NPP instance,
      new_child(instance, fname);
 }
 
-/*****************************************************************************
- * Find and resize the embedded window.
- *****************************************************************************/
-void resize_window(int flags, Display *display, Window *window, int w, int h)
-{
-     Window *useme = NULL;
-     Window root, parent;
-     Window *children = NULL;
-     unsigned int num_children;
-
-     /* tell the child of the stored tree to resize, sadly we don't know
-	the handle of that window, since the helper handles it for us */
-     if (!XQueryTree(display, *window, &root, &parent, &children, &num_children))
-     {
-	  D("Querytree failed!!! using current window\n");
-	  useme = window;
-     }
-     if (children == NULL) {
-	  D("No children!!! using current window\n");
-	  useme = window;
-     } else if (num_children > 1) {
-	  D("Multiple Children, Not Resizing");
-	  XFree(children);
-	  return;
-     } else {
-	  /* use the first child */
-	  useme = &children[0];
-     }
-
-     if (flags & H_FILL) {
-	  fill_window(display, *useme, w, h);
-     } else if (flags & H_MAXASPECT) {
-	  max_aspect_window(display, *useme, w, h);
-     }
-
-     XFree(children);
-}
-
 /*****************************************************************************
  * Called when initializing or resizing the window.
  *****************************************************************************/
@@ -1127,34 +1150,34 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
 
      if (!instance)
 	  return NPERR_INVALID_INSTANCE_ERROR;
-  
+ 
      if (!window)
 	  return NPERR_NO_ERROR;
-  
+ 
      if (!window->window)
 	  return NPERR_NO_ERROR;
 
      if (!window->ws_info)
 	  return NPERR_NO_ERROR;
-
+     
      THIS->display = ((NPSetWindowCallbackStruct *)window->ws_info)->display;
      THIS->displayname = XDisplayName(DisplayString(THIS->display));
      THIS->windata = *window;
-
+     
      if (THIS->mms)
      {
 	  new_child(instance, THIS->mms);
 	  free(THIS->mms);
 	  THIS->mms = NULL;
-     }
-     
-     if ((THIS->flags & H_FILL) || (THIS->flags & H_MAXASPECT))
-     {
-	  resize_window(THIS->flags, THIS->display, (Window *)&window->window,
-			window->width, window->height);
+	  return NPERR_NO_ERROR;
      }
 
-     usleep(4000);
+     if (THIS->fd != -1)
+     {
+	  D("Writing WIN to fd %d\n", THIS->fd);
+	  write(THIS->fd, (char *)window, sizeof(*window));
+	  usleep(4000);
+     }
 
      return NPERR_NO_ERROR;
 }
